import os
import subprocess
from typing import List
import logging

import psutil

PROCESS_NAME = "citrixagent.exe"

log = logging.getLogger(__name__)


def start_citrix_agent():
    current_file_path = os.path.dirname(os.path.realpath(__file__))

    cmd = [os.path.join(current_file_path, "citrixagent", PROCESS_NAME)]
    log.info(f"Attempting to execute command '{' '.join(cmd)}'")
    p = subprocess.Popen(cmd, creationflags=0x00000008)
    log.info(f"Successfully started the process, PID: {p.pid}")


def restart_citrix_agent():
    log.info(f"Restarting the process {PROCESS_NAME}")
    kill_processes(PROCESS_NAME)
    start_citrix_agent()


def check_if_citrix_agent_running():
    log.info(f"Checking if {PROCESS_NAME} is running")
    if not get_processes(PROCESS_NAME):
        log.info(f"No {PROCESS_NAME} found, attempting to start it")
        start_citrix_agent()


def kill_citrix_agent():
    kill_processes(PROCESS_NAME)


def get_processes(name_filter: str) -> List[psutil.Process]:
    """
    Returns the list of process that match the name_filter string exactly
    :param name_filter: The name of the process
    :return: A list of psutil.Process instances
    """
    process_list = []
    for p in psutil.process_iter(attrs=["name"], ad_value="unknow"):
        if p.name() == name_filter:
            log.info(f"Found process '{name_filter}': {p}")
            process_list.append(p)
    return process_list


def kill_processes(name_filter: str) -> bool:
    """
    Kills all process that match the name_filter string exactly.
    :param name_filter: Process name to kill
    :return: None
    """
    for process in get_processes(name_filter):
        # At first, attempt a SIGTERM
        log.info(f"Attempting to cleanly terminate '{process}'")
        try:
            process.terminate()
            gone, alive = psutil.wait_procs([process], timeout=10)
        except psutil.AccessDenied:
            log.error(
                f"Could not kill {process}. Is it running as a different user than {os.getlogin()}?"
            )
            return False
        if alive:
            try:
                # If the process is alive after 10 seconds, SIGKILL
                log.info(f"Attempting to forcefully terminate '{process}'")
                process.kill()
            except psutil.NoSuchProcess:
                pass
        gone, alive = psutil.wait_procs([process], timeout=10)
        if alive:
            # If we get here something is wrong, we cannot kill the process
            log.error(f"Could not kill the process '{process}'")
            return False
        else:
            log.info(f"Terminated the process '{process}'")
            return True
